# WWL-regionaal
# Wageningen Environmental Research
# waterwijzerlandbouw@wur.nl
#------------------------------------------------

# set start of program
TIMPRGSTART <- Sys.time()

# ---- arguments ----

# set modus
test <- FALSE

# load arguments
if (!test) {
  command_args <- commandArgs(trailingOnly = TRUE)
  if (length(command_args) != 1) {
    stop("wrong usage of script, arguments should be:\n- ctrl")
  }
} else {
  command_args <- NULL
  command_args[1] <- "control_regionaal.inp"
  setwd("d:/Regionaal")
}

# set arguments
ctrl <- command_args[1]

# ---- load libraries ----

source("Tools/R/Libraries.R")

sign_open <- "\\{\\{"
sign_close <- "\\}\\}"

message("\nProgram R-PROG started...\n")

# ---- pre-process ----

# set directories
dir_run <- get_dir(file = ctrl, item = "DIRRUN", create = FALSE)

# set database
file_sql <- get_record(file = ctrl, item = "FILSQL")

# set run_id
run_id <- get_record(file = ctrl, item = "RUNID")
if (run_id == "ALL") {
  run_id <- filter_run_id_SQL(file_sql = file_sql, scenario_id = "direct")
} else {
  run_id <- string_sequence(string = run_id)
}

# soil
type <- get_record(file = ctrl, item = "TYPE", opt = c("BODEM", "BOFEK2012", "BOFEK2020"))

# create figure
analyse <- FALSE
record <- get_record(file = ctrl, item = "CRTFIG", opt = c("Yes", "No"), item_exists = FALSE)
if (!is.null(record)) analyse <- ifelse(record == "Yes", TRUE, FALSE)

# ---- main-process ----

# extract key_id from run-directory
key_id <- get_variable(text = dir_run)

# loop over run_id
pb <- progress_bar$new(total = length(run_id), format = "analyse SWAP :percent (run: :what)", clear = TRUE)
err_id <- NULL
err_id_pot <- NULL
for (s_run_id in run_id) {

  # set progress
  pb$tick(tokens = list(what = s_run_id))

  # determine potential run
  run_info <- get_run_info_SQL(run_id = s_run_id, file_sql = file_sql)
  run_id_pot <- filter_run_id_SQL(file_sql = file_sql, meteo_id = run_info$meteo_id, rain_id = run_info$rain_id, soil_id = run_info$soil_id, prof_id = run_info$prof_id, crop_id = run_info$crop_id, scenario_id = "indirect")

  # set run-directory
  dir_tmp <- dir_run
  if (!is.null(key_id)) {
    run_info <- get_run_info_SQL(file_sql = file_sql, run_id = s_run_id)
    for (s_key_id in key_id) {
      dir_tmp <- str_replace(string = dir_tmp, pattern = str_c(sign_open, s_key_id, sign_close), replacement = as.character(run_info[[s_key_id]]))
    }
  }

  # extract simulation results
  dir_tmp <- str_c(dir_tmp, "/ana_", formatC(x = s_run_id, format = "d", width = 9, flag = "0"))
  extract_run_id(run_id = c(s_run_id, run_id_pot), dir_run = dir_run, dir_out = dir_tmp)

  # analyse run
  file_dir <- str_c(dir_tmp, "/run_", formatC(x = s_run_id, format = "d", width = 9, flag = "0"), "/swap.swp")
  file_ind <- str_c(dir_tmp, "/run_", formatC(x = run_id_pot, format = "d", width = 9, flag = "0"), "/swap.swp")

  # check if results are available
  success <- TRUE
  if (!file_exists(file_dir, quiet = TRUE)) {
    success <- FALSE
    err_id <- c(err_id, s_run_id)
  }
  if (!file_exists(file_ind, quiet = TRUE)) {
    success <- FALSE
    err_id_pot <- c(err_id_pot, run_id_pot)
  }

  # proceed if possible
  if (success) {
  
    # create harvest info
    create_harvestinfo(file_swp = file_dir, soil_id = run_info$soil_id, type = type)
    create_harvestinfo(file_swp = file_ind, soil_id = run_info$soil_id, type = type)

    # create figures
    if (analyse) {
      analyse_gwl(file_swp = file_dir, dir_tmp)
      analyse_harvest(file_ind = file_ind, file_dir = file_dir, dir_out = dir_tmp)
      path <- dir_ls(path = dir_tmp, glob = "*.png")
      new_path <- str_c(path_dir(dir_tmp), "/", str_replace_all(string = path_file(path), pattern = ".png", replacement = str_c("_", formatC(x = s_run_id, format = "d", width = 9, flag = "0"), ".png")))
      file_move(path = path, new_path = new_path)
    }

    # get harvest and stresses
    db_plot <- collect_data_plot(file_ind = file_ind, file_dir = file_dir, quiet = TRUE)
    db_plot <- add_period_plot(db_plot = db_plot)

    # write output
    db_plot <- db_plot %>%
      mutate_at(c("ghg", "glg", "irrig", "hrvpotbio", "hrvpotvem", "hrvpotdve"), round, digits = 0) %>%
      mutate_at(c("dmgtot", "dmgind", "dmgdir", "dmgwet", "dmgdry","dmgsol"), round, digits = 1) %>%
      select(period, crop, ghg, glg, irrig, hrvpotbio, hrvpotvem, hrvpotdve, dmgtot, dmgind, dmgdir, dmgwet, dmgdry, dmgsol)
    file <- str_c(dir_run, "/ana_", formatC(x = s_run_id, format = "d", width = 9, flag = "0"), ".csv")
    write_csv(x = db_plot, file = file, progress = FALSE, quote = "none")
  }
  
  # delete temporary directory
  if (file_exists(dir_tmp, quiet = TRUE)) dir_delete(path = dir_tmp)

}

# message to screen
if (!is.null(err_id)) {
  message("missing runs for direct effects:")
  message(str_c("- direct run: ", err_id, "\n"))
}
if (!is.null(err_id_pot)) {
  message("missing runs for indirect effects:")
  message(str_c("- indirect run: ", err_id_pot, "\n"))
}


# ---- return part of procedure ----

TIMPRGEND <- Sys.time()
TIMPRGCALC <- as.numeric(difftime(time1 = TIMPRGEND, time2 = TIMPRGSTART, units = "secs"))
message(paste0("\nProgram R-PROG successfully ended in ", floor(TIMPRGCALC / 60), " minutes and ", ceiling(TIMPRGCALC %% 60), " seconds"))
q(save = "no")
